 
#ifndef INTLATTICE_H
#define INTLATTICE_H
 
#include "latcommon/Types.h"
#include "latcommon/Const.h"
#include "latcommon/Base.h"
#include "latcommon/Normalizer.h"
#include "latcommon/CoordinateSets.h"
#include <string>


namespace LatCommon {

class IntLattice {
public:

   IntLattice (const MScal & m, int k, int maxDim, NormType norm = L2NORM);


   IntLattice (const IntLattice & Lat);


   IntLattice & operator= (const IntLattice & Lat);


   virtual ~IntLattice();


   void copy (const IntLattice & lattice);


   int getDim() const   { return m_v.getDim(); }  


   void setDim (int d);


   int getMaxDim() const   { return m_v.getMaxDim(); }  


   virtual void incDim();


   virtual void buildBasis (int d);


   NormType getNorm() const   { return m_v.getNorm(); }  


   MScal getM() const   { return m_m; }  


   NScal getM2() const   { return m_m2; }  


   int getOrder() const   { return m_order; }  


   Normalizer * getNormalizer (NormaType norma, int alpha);


   virtual std::string toStringCoef() const;


   Base & getPrimalBasis()   { return m_v; }  


   Base & getDualBasis()   { return m_w; }  


   void calcLgVolDual2 (bool dualF);


   double getLgVolDual2 (int i) const   { return m_lgVolDual2[i]; }  


   bool getXX (int i) const   { return m_xx[i]; }  


   void setXX (bool val, int i)   { m_xx[i] = val; }  


   void sort (int d);


   void permute (int i, int j);


   void write (const char *filename) const;


   void trace (char* msg);


   void write (int flag);


   void dualize();


   bool checkDuality();


   bool baseEquivalence (IntLattice & Lat);


   void buildProjection (IntLattice* lattice, const Coordinates & proj);



protected:

   void init();


   void kill();


   int m_order;


   MScal m_m;
   NScal m_m2;


   double m_lgm2;


   Base m_v;


   Base m_w;


   double *m_lgVolDual2;


   bool *m_xx;
   MScal m_t1, m_t2, m_t3;
   BMat m_vSI;
   Base m_vTemp;

};

}
 
#endif
 

