 
#ifndef REDUCER_H
#define REDUCER_H
  
#include "latcommon/Types.h"
#include "latcommon/Const.h"
#include "latcommon/UtilLC.h"
#include "latcommon/Base.h"
#include "latcommon/IntLattice.h"
#include <fstream>
#include <sstream>
#include <vector>


namespace LatCommon {

class Reducer {
public:

   static const long SHORT_DIET = 1000;


   static const long SHORT_LLL = 1000;


   static const long MINK_LLL = 500000;


   static const long MAX_PRE_RED = 1000000;


   static long maxNodesBB;


   static bool PreRedDieterSV;
   static bool PreRedLLLSV;
   static bool PreRedLLLRM;


   Reducer (IntLattice & lat);


   Reducer (const Reducer & red);


   ~Reducer ();


   Reducer & operator= (const Reducer & red);


   void copy (const Reducer & red);


   void transformStage3 (std::vector<long> & z, int & k);


   bool calculCholeski (RVect & DC2, RMat & C0);


   bool tryZ  (int j, int i, int Stage, bool & smaller, Base & WTemp);


   bool tryZ0 (int j, bool & smaller);


   bool shortestVector (NormType norm);


   bool shortestVectorDieter (NormType norm);


   bool redBB (int i, int d, int Stage, bool & smaller);


   bool redBB0 (NormType norm);


   void preRedDieter (int d);


   bool redDieter (NormType norm);


   void redLLL (double fact, long maxcpt, int dim);


   bool reductMinkowski (int d);


   void pairwiseRedPrimal (int i, int d);


   void pairwiseRedDual (int i);


   NScal getMinLength ()   {
      if (m_lat->getNorm() == L2NORM)
         return sqrt(m_lMin2);
      else return m_lMin; }  


   void setBoundL2 (const NVect & V, int dim1, int dim2);


   void trace (char *mess);



private:

   IntLattice* m_lat;


   void permuteGramVD (int i, int j, int n);


   void calculCholeski2LLL (int n, int j);


   void calculCholeski2Ele (int i, int j);


   void miseAJourGramVD (int j);


   void calculGramVD ();


   void reductionFaible (int i, int j);


   NVect m_BoundL2;


   BScal m_bs;
   BVect m_bv;            // Saves shorter vector in primal basis
   BVect m_bw;            // Saves shorter vector in dual basis

   NScal m_lMin, m_lMin2;
   NScal m_ns;
   NVect m_nv; 

   RScal m_rs;
   RVect m_zLR, m_n2, m_dc2;
   RMat m_c0, m_c2, m_cho2, m_gramVD;
   int *m_IC;             // Indices in Cholesky 
   
   std::vector<long> m_zLI;
   std::vector<long> m_zShort;
   long m_countNodes;     // Counts number of nodes in the branch-and-bound tree
   long m_countDieter;    // Counts number of attempts since last successful
                          // Dieter transformation
   long m_cpt;            // Counts number of successes in pre-reduction
                          // transformations
   bool m_foundZero;      // = true -> the zero vector has been found
// long m_BoundL2Count;   // Number of cases for which the reduction stops
                          // because any vector is shorter than L2 Bound.

};

}

 
#endif // REDUCER_H
 

