 
#ifndef UTILLC_H
#define UTILLC_H
 
#include <string>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <vector>
#include <set>
#include <map>
#include <cmath>
#include <cstdlib>
#include "latcommon/Types.h"
#include "latcommon/Const.h"

#ifdef WITH_NTL
#include "NTL/tools.h"
#include "NTL/ZZ.h"
#endif


namespace LatCommon {


void MyExit (int status, std::string msg);



extern const unsigned long TWO_EXP[];



template <typename T>
inline void swap9 (T & x, T & y)   { T t = x; x = y; y = t; }  



#ifndef WITH_NTL
template <typename A, typename B>
void conv (A & x, const B & y)  { x = y; } 
#endif


inline void clear (double & x)  { x = 0; }  
inline void clear (long & x)  { x = 0; }  



inline void set9 (long & x)  
{
    x = 1;
} 
#ifdef WITH_NTL
inline void set9 (NTL::ZZ & x)  
{
    NTL::set(x);
}
 
#endif



inline bool IsZero (const long & x)  
{
    return x == 0;
}
 


template <typename Int>
inline void Quotient (const Int & a, const Int & b, Int & q) 
{
    q = a/b;
}
 

#ifdef WITH_NTL
   inline void Quotient (const NTL::ZZ & a, const NTL::ZZ & b, NTL::ZZ & q) 
{
    NTL::ZZ r;
    DivRem (q, r, a, b);
    if (q < 0 && r != 0)
       ++q;
}
 
#endif


#if 0
template <typename Real, typename T>
inline void Modulo (const Real & a, const T & b, Real & r) 
{
    std::cout << "Modulo Real non testé" << std::endl;
    exit(1);
    Real br;
    br = b;
    r = fmod (a, br);
    if (r < 0) {
        if (b < 0)
            r -= br;
        else
            r += br;
    }
} 
#endif

inline void Modulo (const long & a, const long & b, long & r) 
{
    r = a % b;
    if (r < 0) {
        if (b < 0)
            r -= b;
        else
            r += b;
    }
}
 

#ifdef WITH_NTL
   inline void Modulo (const NTL::ZZ & a, const NTL::ZZ & b, NTL::ZZ & r) 
{
    r = a % b;
    if (r < 0)
        r -= b;
}
 
#endif



template <typename Real>
inline void Divide (Real & q, Real & r, const Real & a, const Real & b) 
{
    q = a / b;
    conv (q, trunc(q));
    r = a - q * b;
}
 


inline void Divide (long & q, long & r, const long & a, const long & b) 
{
    ldiv_t z = ldiv (a, b);
    q = z.quot;      // q = a / b;
    r = z.rem;       // r = a % b;
}
 

#ifdef WITH_NTL
inline void Divide (NTL::ZZ & q, NTL::ZZ & r, const NTL::ZZ & a,
                    const NTL::ZZ & b) 
{
    DivRem (q, r, a, b);
    if (q < 0 && r != 0) {
        ++q;
        r -= b;
    }
}
 
#endif



template <typename Real>
inline void DivideRound (const Real & a, const Real & b, Real & q) 
{
    q = a/b;
    q = floor(q + 0.5);
} 

inline void DivideRound (const long & a, const long & b, long & q) 
{
    bool neg;
    if ((a > 0 && b < 0) || (a < 0 && b > 0))
       neg = true;
    else
       neg = false;
    long x = abs(a);
    long y = abs(b);
    ldiv_t z = ldiv (x, y);
    q = z.quot;
    long r = z.rem;
    r <<= 1;
    if (r > y)
       ++q;
    if (neg)
       q = -q;
} 

#ifdef WITH_NTL
inline void DivideRound (const NTL::ZZ & a, const NTL::ZZ & b, NTL::ZZ & q) 
{
    bool s = false;
    if ((a > 0 && b < 0) || (a < 0 && b > 0))
       s = true;
    NTL::ZZ r, x, y;
    x = abs (a);
    y = abs (b);
    //****** ATTENTION: bug de NTL: DivRem change le signe de a quand a < 0.
    DivRem (q, r, x, y);
    LeftShift (r, r, 1);
    if (r > y)
       ++q;
    if (s)
       q = -q;
}
 
#endif


void Euclide (const MScal & a, const MScal & b, MScal & C, MScal & D,
              MScal & E, MScal & F, MScal & G);


long gcd (long a, long b);



template <typename Scal>
inline Scal abs (Scal x)  {
   if (x < 0) return -x;  else return x;
}
 



template <typename Real>
inline Real Round (Real x) 
{
    return floor (x + 0.5);
}
 



bool IsPrime (unsigned long n);


template <typename Vect>
inline void SetZero (Vect & A, int d) 
{
    for (int i = 0; i <= d; i++)
        A[i] = 0;
}
 



template <typename Vect, typename Scal>
inline void ModifVect (Vect & A, Vect & B, const Scal & x, int n) 
{
    MScal y;
    conv (y, x);
    for (int i = 1; i <= n; i++)
       A[i] = A[i] + B[i]*y;
}
 



template <typename Vect, typename Scal>
inline void ModifVect (MVect & A, Vect & B, const Scal & x, int n) 
{
    MScal y;
    conv (y, x);
    for (int i = 1; i <= n; i++)
       A[i] = A[i] + B[i]*y;
}
 



template <typename V, typename Scal>
inline void ProdScal (const V & A, const V & B, int n, Scal & D) 
{
    // Le produit A[i] * B[i] peut déborder, d'où conv.
    MScal C;   C = 0;
    for (int i = 1; i <= n; i++)
        C += A[i] * B[i];
    conv (D, C);
} 


template <typename Scal>
inline void ProdScal (const BVect & A, const BVect & B, int n, Scal & D) 
{
    // Le produit A[i] * B[i] peut déborder, d'où conv.
    MScal C;   C = 0;
    for (int i = 1; i <= n; i++)
        C += A[i] * B[i];
    conv (D, C);
} 



template <typename Vect>
inline void ChangeSign (Vect & A, int n) 
{
    for (int i = 1; i <= n; i++) A[i] = -A[i];
}
 



inline long GCD2vect (std::vector<long> V, int k, int n) 
{
    int i = k + 1;
    long r, d, c;
    d = labs (V[k]);
    while (d != 1 && i <= n) {
        c = labs (V[i]);
        while (c) {
            r = d % c;
            d = c;
            c = r;
        }
        ++i;
    }
        return d;
}
 



template <typename Vect, typename Scal>
inline void CalcNorm (const Vect & V, int n, Scal & S, NormType norm) 
{
    Scal y;
    S = 0;
    switch (norm) {
        case L1NORM:
            for (int i = 1; i <= n; i++) {
                conv (y, V[i]);
                S += abs(y);
            }
            break;

        case L2NORM:
            for (int i = 1; i <= n; i++) {
                conv (y, V[i]);
                S += y*y;
            }
            //S = sqrt(S);
            break;

        case SUPNORM:
            for (int i = 1; i <= n; i++) {
                conv (y, abs(V[i]));
                if (y > S)
                    S = y;
            }
            break;

        case ZAREMBANORM:
            S = 1.0;
            for (int i = 1; i <= n; i++) {
                conv (y, abs(V[i]));
                if (y > 1.0)
                   S *= y;
            }
            break;

        default:
            ;
    }
}
 


template <typename Matr>
bool CheckTriangular (const Matr & A, int dim, const MScal & m)  
{
   for (int i = 2; i <= dim; i++) {
      for (int j = 1; j < i; j++) {
         if (A(i,j) % m != 0) {
            std::cout << "******  CheckTriangular failed for element A[" << i <<
                 "][" << j << "]" << std::endl;
            return false;
         }
      }
    }
    return true;
} 



template <typename Matr>
void Triangularization (Matr & W, Matr & V, int lin, int col,
                        const MScal & m)  
{
   MScal T1, T2, T3, T4, T5, T6, T7, T8;

   for (int j = 1; j <= col; j++) {
      for (int i = 1; i <= lin; i++)
         Modulo (W(i,j), m, W(i,j));
      int r = 1;
      while (r < lin) {
         while (IsZero (W(r,j)) && r < lin)
            ++r;
         if (r < lin) {
            int s = r + 1;
            while (IsZero (W(s,j)) && s < lin)
               ++s;
            if (!IsZero (W(s,j))) {
               Euclide (W(r,j), W(s,j), T1, T2, T3, T4, W(s,j));
               clear (W(r,j));
               for (int j1 = j + 1; j1 <= col; j1++) {
                  T5 = T1 * W(r,j1);
                  T6 = T2 * W(s,j1);
                  T7 = T3 * W(r,j1);
                  T8 = T4 * W(s,j1);
                  W(s,j1) = T5 + T6;
                  Modulo (W(s,j1), m, W(s,j1));
                  W(r,j1) = T7 + T8;
                  Modulo (W(r,j1), m, W(r,j1));
               }
            } else {
               for (int j1 = j; j1 <= col; j1++) {
                  std::swap (W(r,j1), W(s,j1));
               }
            }
            r = s;
         }
      }
      if (IsZero (W(lin,j))) {
         for (int j1 = 1; j1 <= col; j1++) {
            if (j1 != j)
               clear (V(j,j1));
            else
               V(j,j1) = m;
         }
      } else {
         Euclide (W(lin,j), m, T1, T2, T3, T4, V(j,j));
         for (int j1 = 1; j1 < j; j1++)
            clear (V(j,j1));
         for (int j1 = j + 1; j1 <= col; j1++) {
            T2 = W(lin,j1) * T1;
            Modulo (T2, m, V(j,j1));
         }
         Quotient (m, V(j,j), T1);
         for (int j1 = j + 1; j1 <= col; j1++) {
            W(lin,j1) *= T1;
            Modulo (W(lin,j1), m, W(lin,j1));
         }
      }
   }
//   CheckTriangular (V, col, m);
} 



template <typename Matr>
void CalcDual (const Matr & A, Matr & B, int d, const MScal & m)  
{
   for (int i = 1; i <= d; i++) {
      for (int j = i + 1; j <= d; j++)
         clear (B(i,j));
// Dans l'original, c'est Quotient pour Lac et DivideRound pour non-Lac ??
//        Quotient(m, A(i,i), B(i,i));
      DivideRound (m, A(i,i), B(i,i));
      for (int j = i - 1; j >= 1; j--) {
         clear (B(i,j));
         for (int k = j + 1; k <= i; k++) {
            B(i,j) += A(j,k) * B(i,k);
         }
         if (B(i,j) != 0)
            B(i,j) = -B(i,j);
// Dans l'original, c'est Quotient pour Lac et DivideRound pour non-Lac ??
//         Quotient(B(i,j), A(j,j), B(i,j));
         DivideRound (B(i,j), A(j,j), B(i,j));
      }
   }
} 


template <typename Vect>
std::string toString (const Vect & A, int c, int d, const char* sep) 
{
    std::ostringstream out;
    out << "[";
    for (int i = c; i < d; i++)
        out << A[i] << sep;
    out << A[d] << "]";
    return out.str ();
}
 



template <typename Vect>
inline std::string toString (const Vect & A, int c, int d) 
{
    return toString (A, c, d, "  ");
}
 



template <typename Vect>
std::string toString (const Vect & A, int d) 
{
   std::ostringstream ostr;
   ostr << "[";
   for (int i = 1; i <= d; i++) {
      ostr << std::setprecision(2) << std::setw(3) << A[i] <<
              std::setw(2) << "  ";
   }
   ostr << "]";
   return ostr.str();
}
 



template <class T1, class T2>
std::ostream & operator<< (std::ostream & out, const std::pair<T1,T2> & x)  
{
   out << "(" << x.first << "," << x.second << ")";
   return out;
} 



template <class T, class A>
std::ostream & operator<< (std::ostream & out, const std::vector<T,A> & x)  
{
   out << "[";
   typename std::vector<T,A>::const_iterator it = x.begin();
   if (it != x.end()) {
      out << *it;
      while (++it != x.end())
         out << ", " << *it;
   }
   out << "]";
   return out;
} 



template <class K, class C, class A>
std::ostream & operator<< (std::ostream & out, const std::set<K,C,A> & x)  
{
   out << "{";
   typename std::set<K,C,A>::const_iterator it = x.begin();
   if (it != x.end()) {
      out << *it;
      while (++it != x.end())
         out << ", " << *it;
   }
   out << "}";
   return out;
} 



template <class K, class T, class C, class A>
std::ostream & operator<< (std::ostream & out, const std::map<K,T,C,A> & x)  
{
   out << "{";
   typename std::map<K,T,C,A>::const_iterator it = x.begin();
   if (it != x.end()) {
      out << it->first << "=>" << it->second;
      while (++it != x.end())
         out << ", " << it->first << "=>" << it->second;
   }
   out << "}";
   return out;
} 

}
 
#endif
 

