 
#ifndef UTILLM_H
#define UTILLM_H
 
#include <cstdlib>
#include <cmath>
#include <iostream>
#include <iomanip>
#include <sstream>
#include <string>

#include "NTL/tools.h"
#include "NTL/ZZ.h"
#include "latcommon/vec_lzz.h"
#include "latcommon/Types.h"
#include "latcommon/Const.h"



namespace LatMRG {

const double MAX_LONG_DOUBLE = 9007199254740992.0;


inline long IsOdd (const long & x)   {
    return x & 1;
} 


inline long power (long p, long i)  
{
   return NTL::power_long (p, i);
}
 



inline void power2 (long & z, long i)  
{
   z = NTL::power_long (2, i);
}
 



inline double mysqrt (double x)  
{
    if (x < 0.0)
       return -1.0;
    return sqrt (x);
}
 



inline double SqrRoot (double x)  
{
    return sqrt (x);
}
 



template <typename T>
inline double Log2 (const T & x) 
{
   return log (x) / 0.6931471805599453094;
}
 



inline double Log2 (long x) 
{
   return log ((double)x) / 0.6931471805599453094;
}
 



template <typename T>
inline long sign (const T & x) 
{
    if (x > 0) return 1; else if (x < 0) return -1; else return 0;
}
 



inline void div (long & a, const long & b, const long & d) 
{
    a = b/d;
}
 


template <typename Real>
inline void CreateVect (Real* & A, int d) 
{
    A = new Real[1 + d];
    for (int i = 0; i <= d; i++)
        A[i] = 0;
}
 



template <typename Real>
inline void DeleteVect (Real* & A) 
{
    delete[] A;
//    A = 0;
}
 



template <typename Vect>
inline void CreateVect (Vect & A, int d) 
{
    A.SetLength (1 + d);
    // aaa clear (A);
}
 



template <typename Vect>
inline void DeleteVect (Vect & A) 
{
    A.kill ();
}
 



template <typename Real>
inline void SetZero (Real* A, int d) 
{
    for (int i = 0; i <= d; i++)
        A[i] = 0;
}
 



template <typename Real>
inline void SetValue (Real* A, int d, const Real & x) 
{
    for (int i = 0; i <= d; i++)
        A[i] = x;
}
 



inline void Invert (const MVect & A, MVect & B, int n) 
{
    conv(B[n], 1);
    for(int i = 0; i < n; i++){
       B[i] = -A[n - i];
    }
}
 



template <typename Vect>
inline void CopyVect (const Vect & A, Vect & B, int n) 
{
    for (int k = 0; k <= n; k++)  B[k] = A[k];
}
 



template <typename Xcal, typename Scal>
inline void ModifVect (Xcal *A, const Xcal *B, Scal x, int n) 
{
    Xcal a;
    conv (a, x);
    for (int i = 1; i <= n; i++)
        A[i] += B[i]*a;
}
 


template <typename Real>
inline void CreateMatr (Real** & A, int d) 
{
    A = new Real *[1 + d];
    for (int i = 0; i <= d; i++) {
        A[i] = new Real[1 + d];
        for (int j = 0; j <= d; j++)
            A[i][j] = 0; }
}
 



template <typename Real>
inline void DeleteMatr (Real** & A, int d) 
{
    for (int i = d; i >= 0; --i)
        delete[] A[i];
    delete[] A;
 //   A = 0;
}
 



template <typename Real>
inline void CreateMatr (Real** & A, int line, int col) 
{
    A = new Real *[1+line];
    for (int i = 0; i <= line; i++) {
        A[i] = new Real[1+col];
        for (int j = 0; j <= col; j++)
            A[i][j] = 0;
    }
}
 



template<typename Real>
inline void DeleteMatr (Real** & A, int line, int col) 
{
    for (int i = line; i >= 0; --i)
        delete [] A[i];
    delete [] A;
//    A = 0;
}
 



inline void CreateMatr (MMat& A, int d) 
{
    A.SetDims (1 + d, 1 + d);
    //clear (A);
}
 



inline void CreateMatr (MMatP & A, int d) 
{
    A.SetDims (1 + d, 1 + d);   clear (A);
}
 



inline void CreateMatr (MMat& A, int line, int col) 
{
    A.SetDims(1 + line, 1 + col);
    //clear (A);
}
 



inline void CreateMatr (MMatP & A, int line, int col) 
{
    A.SetDims (1 + line, 1 + col);   clear (A);
}
 



inline void DeleteMatr (MMat& A) 
{
    A.kill ();
}
 



inline void DeleteMatr (MMatP & A) 
{
    A.kill ();
}
 



template <typename Matr>
inline void CopyMatr (const Matr & A, Matr & B, int n) 
{
    for (int i = 1; i <= n; i++)
        for (int j = 1; j <= n; j++)
            B[i][j] = A[i][j];
}
 



template <typename Matr>
inline void CopyMatr (const Matr & A, Matr & B, int line, int col) 
{
    for (int i = 1; i <= line; i++)
        for (int j = 1; j <= col; j++)
            B[i][j] = A[i][j];
}
 



template <typename MatT>
std::string toStr (const MatT & mat, int d1, int d2)  
{
   std::ostringstream ostr;
   for (int i = 1; i <= d1; i++) {
      ostr << "[";
      for (int j = 1; j <= d2; j++) {
         ostr << std::setprecision (2) << std::setw (6) << mat[i][j] <<
            std::setw (2) << " ";
      }
      ostr << "]\n";
   }
   return ostr.str ();
} 


}     // namespace LatMRG

namespace NTL {

inline void conv (long & l, const char* c)   {
    l = strtol(c, (char **) NULL, 10);
} 



inline void conv (double & r, const char* c)   {
    r = strtod(c, (char **) NULL);
} 


}     // namespace
 
#endif
 

